package de.hft.stuttgart.citydoctor2.datastructure;

import de.hft.stuttgart.citydoctor2.check.Check;
import de.hft.stuttgart.citydoctor2.check.CheckError;
import de.hft.stuttgart.citydoctor2.check.CheckId;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import de.hft.stuttgart.citydoctor2.utils.CopyHandler;
import de.hft.stuttgart.citydoctor2.utils.Copyable;
import org.citygml4j.core.util.geometry.GeometryFactory;

import java.io.Serial;
import java.util.ArrayList;
import java.util.List;

public class BuildingUnit extends AbstractBuildingSubdivision {

    @Serial
    private static final long serialVersionUID = -2931788311665810322L;

    private final List<Storey> storeys = new ArrayList<>();

    @Override
    public Copyable createCopyInstance() {
        return new BuildingUnit();
    }


    @Override
    public void unsetGmlGeometries() {
        super.unsetGmlGeometries();
        for (Storey storey : storeys) {
            storey.unsetGmlGeometries();
        }
    }

    @Override
    public void reCreateGeometries(GeometryFactory factory, ParserConfiguration config) {
        super.reCreateGeometries(factory, config);
        for (Storey storey : storeys) {
            storey.reCreateGeometries(factory, config);
        }
    }

    @Override
    public void accept(Check c) {
        super.accept(c);
        for (Storey storey : storeys) {
            storey.accept(c);
        }
    }

    @Override
    public void collectContainedErrors(List<CheckError> errors) {
        super.collectContainedErrors(errors);
        for (Storey storey : storeys) {
            storey.collectContainedErrors(errors);
        }
    }

    @Override
    public void clearAllContainedCheckResults() {
        super.clearAllContainedCheckResults();
        for (Storey storey : storeys) {
            storey.clearAllContainedCheckResults();
        }
    }

    @Override
    public boolean containsError(CheckId checkIdentifier) {
        boolean hasError = super.containsError(checkIdentifier);
        if (hasError) {
            return true;
        }
        for (Storey storey : storeys) {
            if (storey.containsError(checkIdentifier)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public boolean containsAnyError() {
        boolean hasError = super.containsAnyError();
        if (hasError) {
            return true;
        }
        for (Storey storey : storeys) {
            if (storey.containsAnyError()) {
                return true;
            }
        }
        return false;
    }

    @Override
    public void prepareForChecking() {
        super.prepareForChecking();
        for (Storey storey : storeys) {
            storey.prepareForChecking();
        }
    }

    @Override
    public void clearMetaInformation() {
        super.clearMetaInformation();
        for (Storey storey : storeys) {
            storey.clearMetaInformation();
        }
    }

    @Override
    public void collectInstances(CopyHandler handler) {
        super.collectInstances(handler);
        for (Storey storey : storeys) {
            storey.collectInstances(handler);
        }
    }

    @Override
    public void fillValues(Copyable original, CopyHandler handler) {
        super.fillValues(original, handler);
        BuildingUnit originalBu = (BuildingUnit) original;
        for (Storey storey : originalBu.storeys) {
            storeys.add(handler.getCopyInstance(storey));
        }
        this.abs = originalBu.abs;
    }
}
